library DelphiPlug;

{%Icon 'res\Example.ico'}
{%Bmp 'res\DefImage.bmp'}
{$R 'Example.res' 'Example.rc'}

uses
  SysUtils,
  Graphics,
  Windows,
  Controls,
  OptPlugin in 'OptPlugin.pas',
  PropertiesForm in 'PropertiesForm.pas' {PropertiesDlg};

{$E opl}
{$R *.res}

type
  TDelphiPlugin = class(OptimusMiniPlugin)
  public
    constructor Create(hModule: HMODULE; func_id: Integer);

    // OptimusMiniPlugin
    function Paint(hdc: HDC): BOOL; override;
    procedure OnKeyDown; override;
    function GetInfo(index: Integer): LPARAM; override;
    function GetProperty(index: Integer; name, value: PChar): BOOL; override;
    function SetProperty(name, value: PChar): BOOL; override;
    function ShowProperties(hwnd: HWND): BOOL; override;
    function OnEvent(event: Integer): BOOL; override;

  private
    m_hModule: HMODULE; // for resource loading
    m_Color: TColor;
    m_bPaintRequired: bool;
  end;

  POptimusMiniPlugin = ^OptimusMiniPlugin;
  PTDelphiPlugin = ^TDelphiPlugin;

constructor TDelphiPlugin.Create(hModule: HMODULE; func_id: Integer);
begin
  m_hModule := hModule;
  m_Color := clWhite;
end;

// OptimusMiniPlugin implementation

function TDelphiPlugin.Paint(hdc: HDC): BOOL;
var
  canvas: TCanvas;

begin
  Paint := FALSE;

  if m_bPaintRequired then
  begin
    canvas := TCanvas.Create();

    canvas.Handle := hdc;
    canvas.Pen.Color := m_Color;
    canvas.Brush.Color := m_Color;

    canvas.Rectangle(0, 0, IMG_X_SIZE, IMG_Y_SIZE);
    canvas.Free();

    m_bPaintRequired := false;
    Paint := TRUE;
  end;
end;

procedure TDelphiPlugin.OnKeyDown;
begin
  // Action
  Beep(1000,100);
end;

function TDelphiPlugin.GetInfo(index: Integer): LPARAM;
begin
  case index of
    INFO_KEY: GetInfo := LPARAM(PChar('delphi.example.vcl'));
    INFO_NAME: GetInfo := LPARAM(PChar('Delphi Example'));
    INFO_DESC: GetInfo := LPARAM(PChar('My first Delphi plug-in example'));
    INFO_AUTHOR: GetInfo := LPARAM(PChar('Author Name'));
    INFO_EMAIL: GetInfo := LPARAM(PChar('author@email.com'));
    INFO_HOMEPAGE: GetInfo := LPARAM(PChar('http://author.url.com'));

    INFO_ICON16: GetInfo := LPARAM(LoadImage(m_hModule, PChar('BIG_ICON'), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR));
    INFO_ICON32: GetInfo := LPARAM(LoadImage(m_hModule, PChar('BIG_ICON'), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR));
    INFO_DEFIMAGE: GetInfo := LPARAM(LoadImage(m_hModule, PChar('IDB_DEFIMAGE'), IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR));

    INFO_HAS_PROPERTIES: GetInfo := 1;
    else
      GetInfo := 0;
  end;
end;

function TDelphiPlugin.GetProperty(index: Integer; name, value: PChar): BOOL;
begin
  GetProperty := FALSE;

  if index = PLUGIN_PROPERTIES + 0 then
  begin
    StrPLCopy(name, 'Color', MAX_PROPERTY_LENGTH);
    StrPLCopy(value, IntToStr(m_Color), MAX_PROPERTY_LENGTH);
    GetProperty := TRUE;
  end;
end;

function TDelphiPlugin.SetProperty(name, value: PChar): BOOL;
begin
  SetProperty := TRUE;

  if 'Color' = String(name) then
  try
    m_Color := StrToInt(value);
  except
    SetProperty := FALSE;
  end;
end;

function TDelphiPlugin.ShowProperties(hwnd: HWND): BOOL;
var
  PropertiesDlg: TPropertiesDlg;

begin
  ShowProperties := FALSE;
  PropertiesDlg := TPropertiesDlg.CreateParented(hwnd);

  try
    PropertiesDlg.SetColor(m_Color);

    if PropertiesDlg.ShowModal = mrOk then
    begin
      m_Color := PropertiesDlg.m_Color;
      m_bPaintRequired := true;
      ShowProperties := TRUE;
    end;
  finally
    PropertiesDlg.Free;
  end;
end;

function TDelphiPlugin.OnEvent(event: Integer): BOOL;
begin
  case event of
    EVENT_ACTIVATE: m_bPaintRequired := true;
    EVENT_DEACTIVATE: ;
  end;

  OnEvent := TRUE;
end;

// Factory functions for exporting

function Create(hModule: HMODULE; func_id: Integer; Reserved: LPARAM): POptimusMiniPlugin; cdecl;
begin
  Create := Pointer(TDelphiPlugin.Create(hModule, func_id));
end;

procedure Delete(instance: POptimusMiniPlugin); cdecl;
begin
  instance.Free;
end;

exports
 	 Create, Delete;

begin
end.
