unit OptPlugin;

interface

uses
    Windows;  //4 types

const
// device OLED dimensions and buttons count
IMG_X_SIZE = 96;
IMG_Y_SIZE = 96;
MAX_BUTTON = 3;

// string information to be provided by plug-in
INFO_KEY				        = 0;
INFO_NAME				        = 1;
INFO_DESC				        = 2;
INFO_AUTHOR				      = 3;
INFO_EMAIL				      = 4;
INFO_HOMEPAGE			      = 5;
INFO_APPLICATION	     	= 6;
INFO_APPLICATION_CONFIG	= 7;

// integer information to be provided by plug-in
INFO_FUNCTIONS	   = 10;
INFO_DELAY		     = 11;
INFO_MAIN_FUNCTION = 12;

// HICON information to be provided by plug-in
INFO_ICON16		= 20;
INFO_ICON32		= 21;
INFO_DEFIMAGE	= 22;

// information if plug-in has values to be saved in registry
INFO_HAS_PROPERTIES		     = $100-1;
INFO_HAS_SHARED_PROPERTIES = $200-1;

// properties bases. first has PLUGIN_PROPERTIES index, second PLUGIN_PROPERTIES+1, and so on
PLUGIN_PROPERTIES		     = $100;
PLUGIN_SHARED_PROPERTIES = $200;

// these are events plug-in can react on.
EVENT_ACTIVATE	 = 1;
EVENT_DEACTIVATE = 2;

MAX_INFO_STRING     = 256;
MAX_PROPERTY_LENGTH = 256;

type

  OptimusMiniPlugin = class
  public
  	// called every INFO_DELAY ms or three times a second by default.
    // should return true if anything painted, false means previous image should be used.
    // HDC:  has IMG_X_SIZE x IMG_Y_SIZE dimensions
    function Paint(hdc: HDC): BOOL; virtual; stdcall; abstract;

	  // called on keydown from device thread. should return fast.
    procedure OnKeyDown; virtual; stdcall; abstract;

  	// to collect INFO_* values from plug-in
    function GetInfo(index: Integer): LPARAM; virtual; stdcall; abstract;

	  // allows to get properties from plug-in instance
  	// index: from PLUGIN_PROPERTIES and PLUGIN_SHARED_PROPERTIES until false is return.
  	// should return value and name (no longer then MAX_PROPERTY_LENGTH bytes)
    function GetProperty(index: Integer; name, value: PChar): BOOL; virtual; stdcall; abstract;

    // allows to set plug-in instance properties
    // called for every name got with GetProperty with actial value from registry
    function SetProperty(name, value: PChar): BOOL; virtual; stdcall; abstract;

    // shows plug-in properties dialog.
    // should return true if properties has been changed.
    function ShowProperties(hwnd: HWND): BOOL; virtual; stdcall; abstract;

    // plug-in life cicle is OnEvent(ACTIVATE) -> Paint(),Paint().. -> OnEvent(DEACTIVATE)
    // ACTIVATE/DEACTIVATE can be used to allocate resources requied for painting
    // should return TRUE. should return fast.
    function OnEvent(event: Integer): BOOL; virtual; stdcall; abstract;
  end;

implementation

end.
